local initialized = false;

local dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Pathfinder", {
	type = "data source", 
	text = "Nothing to track!", 
	icon = 237385,
	OnClick = function(clickedframe, button)
			ToggleAchievementFrame();
		Pathfinder.Update();
	end,
})

local function Initialize()
	if not Pathfinder.Status then
		Pathfinder.Status = {
			[11446] = { -- Broekn Isles Pathfinder, Part Two
				Completed = false;
				[11190] = { --Broken Isles Pathfinder, Part One
					Completed = false;
					[10672] = { -- Broken Isles Diplomat
						Completed = false;
					}, 
					[11189] = { -- Variety is the Spice of Life
						Completed = false;
					},
					[10994] = { --A Glorious Campaign
						Completed = false;
					},
					[11157] = { --Loremaster of Legion
						Completed = false;
						[10059] = { --Ain't No Mountain High Enough
							Completed = false;
						},
						[10763] = { -- Azsuna Matata
							Completed = false;
						},
						[11124] = { -- Good Suramaritan
							Completed = false;
						},
						[19698] = { -- That's Val'sharah Folks!
							Completed = false;
						},
						[10790] = { -- Vrykul Story, Bro
							Completed = false;
						},
					},
					[11188] = { -- Broken Isles Explorer
						Completed = false;
						[10665] = { -- Explore Azsuna
							Completed = false;
						},
						[10667] = { -- Explore Highmountain
							Completed = false;
						},
						[10669] = { -- Explore Suramar
							Completed = false;
						},
						[10668] = { -- Explore Stormheim
							Completed = false;
						},
						[10666] = { -- Explore Val'sharah
							Completed = false;
						},
					},
				},
				[11544] = { -- Defender of the Broken Isles
					Completed = false;
				}, 
				[11543] = { -- Explore Broken Shore
					Completed = false;
				}, -- Explore Broken Shore
				[11545] = { -- Legionfall Commander
					Completed = false;
				}, 
			},
		}
	end

	Pathfinder.GetColor = function (value)
		value = value * 2;
		local r = (2 - value);
		local g = value;
		local b = 0;
		
		if r > 1 then r = 1 end
		if g > 1 then g = 1 end
		if b > 1 then b = 1 end

		r = string.format("%i", r * 255);
		g = string.format("%i", g * 255);
		b = string.format("%i", b * 255);

		return "ff" .. string.format("%02x", r) .. string.format("%02x", g) .. string.format("%02x", b);
	end

	Pathfinder.Filter = function(id)
		local _, name, _, completed, _, _, _, _, _, icon = GetAchievementInfo(id);
		local quantity = 0
		local required = GetAchievementNumCriteria(id);
		if required > 0 then
			for index = 1, required do
				if select(3, GetAchievementCriteriaInfo(id, index)) == true then
					quantity = quantity + 1;
				end
			end
		end
		return name, completed, icon, quantity, required;
	end


	Pathfinder.Traverse = function(table, key)
		for k, v in pairs(table) do
			if type(v) == "table" then
				table[k] = Pathfinder.Traverse(v, k);
			end
		end
		if key then
			table.Name, table.Completed, table.Icon, table.Quantity, table.Required = Pathfinder.Filter(key);
		end
		return table;
	end

	Pathfinder.GetPercent = function (n)
		local s;
		if n.Completed then
			s = 1;
		elseif n.Required == 0 then
			s = 0;
		else
			s = 1 * (n.Quantity / n.Required);
		end
		return "|c" .. Pathfinder.GetColor(s) .. string.format("%#3.2f%%", s * 100) .. "|r";
	end

	Pathfinder.Tooltip = function (frame, item)
		if type(item) == "table" then
			if item.Name and not item.Completed then
				frame:AddDoubleLine(item.Name, Pathfinder.GetPercent(item));
			end
			for k, v in pairs(item) do
				Pathfinder.Tooltip(frame, v);
			end
		end
	end

	Pathfinder.Update = function()
		Pathfinder.Status = Pathfinder.Traverse(Pathfinder.Status);
	end

	return true;
end

function dataobj:OnTooltipShow()
	self:AddLine("|cffe5cc80Pathfinder - Flying Progression Tracker|r|n|n");
	Pathfinder.Tooltip(self, Pathfinder.Status);
	self:AddLine("|nLeft Click to toggle Percentage or Fractional Display");
	self:AddLine("|nRight Click to toggle Icon or Text Labels")
end

function dataobj:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_NONE");
	GameTooltip:SetPoint("TOP", self, "BOTTOM");
	GameTooltip:ClearLines();
	dataobj.OnTooltipShow(GameTooltip);
	GameTooltip:Show();
end

function dataobj:OnLeave()
	GameTooltip:Hide()
end

local function EventHandler(self, event, ...)
	if event == "VARIABLES_LOADED" then
		if Initialize() then 
			initialized = true;
			print("|cffe5cc80Pathfinder v" .. GetAddOnMetadata("Pathfinder", "Version") .. " Loaded!|r");
		end
	end
	if event == "CRITERIA_EARNED" or "PLAYER_ENTERING_WORLD" and initialized then
		Pathfinder.Update();
	end
end

local EventListener = CreateFrame("FRAME", "Pathfinder");
EventListener:RegisterEvent("VARIABLES_LOADED");
EventListener:RegisterEvent("CRITERIA_EARNED");
EventListener:RegisterEvent("PLAYER_ENTERING_WORLD");
EventListener:SetScript("OnEvent", EventHandler);
